// Prevent console window in addition to Slint window in Windows release builds when, e.g., starting the app via file manager. Ignored on other platforms.
#![cfg_attr(not(debug_assertions), windows_subsystem = "windows")]

mod gui;
use gui::GUIElements;
use rfd::{FileDialog, MessageDialog};
use std::{cell::RefCell, rc::Rc};
use slint::SharedString;

slint::include_modules!();

fn main() -> Result<(), slint::PlatformError> {
    let main_window_ref  = Rc::new(RefCell::new(MainWindow::new()?));
    let gui_elements_ref = Rc::new(RefCell::new(GUIElements::new(main_window_ref.clone())?));

    setup_main_tab(gui_elements_ref.clone());
    setup_file_tab(gui_elements_ref.clone());

    let main_window = main_window_ref.borrow();
    main_window.run()
}

fn setup_main_tab(gui_elements_ref: Rc<RefCell<GUIElements>>) {
    let gui_elements = gui_elements_ref.borrow();

    gui_elements.main_tab.on_move_vram_image(gui_elements_ref.clone(), move |main_tab, _main_window, idx, dx, dy| {
        main_tab.move_vram_image(idx as usize, dx, dy);
    });

    gui_elements.main_tab.on_remove_file(gui_elements_ref.clone(), move |main_tab, _main_window, idx| {
        if idx >= 0 {
            main_tab.remove_vram_file(idx as usize);
        }
    });
}

fn setup_file_tab(gui_elements_ref: Rc<RefCell<GUIElements>>) {
    let gui_elements = gui_elements_ref.borrow();

    gui_elements.file_tab.on_browse_file(gui_elements_ref.clone(), move |main_window, main_tab| {
        let file = FileDialog::new()
            .add_filter("Images (.png; .bmp; .jpeg)", &["png", "bmp", "jpeg"])
            .set_title("Select image file")
            .pick_file();

        if let Some(file) = file {
            if let Some(file_path) = file.to_str() {
                main_window.set_file_tab_browse_path(SharedString::from(file_path));
            }

            if let Result::Err(error) = main_tab.add_new_vram_file(file) {
                MessageDialog::new().set_title("Loading Image failed").set_level(rfd::MessageLevel::Error).set_description(error.to_string()).show();
            }
            main_window.invoke_change_to_main();
        }
    });
}